#include "image.h"
#include <stdint.h>
#include <stdio.h>
#include <strings.h>
#include <string.h>
#include <stdlib.h>
#include "tools.h"

int create_image(image_t *img, int w, int h)
{
	if (img==NULL) return -1;
	if (h<1) return -1;
	if (w<1) return -1;
	if (h>1024*16) return -1;
	if (w>1024*16) return -1;
	img->img=malloc(h*w);
	if (img->img==NULL) return -1;
	img->width=w;
	img->height=h;
	return 0;
}

int free_image(image_t *img)
{
	if (img==NULL) return -1;
	img->width=0;
	img->height=0;
	if (img->img==NULL) return -1;
	free(img->img);
	img->img=NULL;
	return 0;
}


int read_int(char **t)
{
	while ((**t<'0') || (**t>'9')) *t=*t+1;
	int n=0;
	while ((**t>='0') && (**t<='9')) {
		n=n*10+**t-'0';
		*t=*t+1;
	}
	return n;
}

int load_from_file(image_t *img, char *fn)
{
	int res=0;
	FILE *fp;
	fp = fopen(fn,"r");
	if (fp==NULL) return -1;
	//read header
	char buffer[128];
	fread(buffer,sizeof(buffer),1,fp);
	if (strncmp(buffer,"P4",2)!=0) {
		fclose(fp);
		return -1;
	}
	char *token=&(buffer[3]);
	while (*token!='\n') token++;
	//P4 already checked avove
	int w=read_int(&token)/8;
	int h=read_int(&token);
	res=create_image(img,w,h);
	if (res!=0) return res;
	res=fseek(fp,-w*h,SEEK_END);
	res=fread(img->img,w*h,1,fp);
	fclose(fp);
	return 0;
}


void refresh_badge()
{
	putchar(0xff);
	fflush(stdout);
	while (gc()!=BDG_ACK);
}


void write_image_(image_t *img, int pos)
{
	int imglen=img->width*img->height;
	int max=pos-imglen;
	int c=-1;
	int cnt=0;
	while (pos<imglen) {
		if ( (img->img[pos]!=c) || (cnt>=32) ) {
			if ( (c!=-1) && (cnt>0) ) {
				putchar(0x80+cnt-1);
				putchar(c);
			}
			c=img->img[pos];
			cnt=0;
		}
		cnt=cnt+1;
		pos=pos+1;
	}	
	putchar(0x80+cnt-1);
	putchar(c);
	refresh_badge();
}

//writes an image to the badge
void write_image(image_t *img)
{
	if (img==NULL) return;
	putchar(0xfd); //set position;
	putchar(0x00);
	putchar(0x00);
write_image_(img,0);
return;
	int len=img->width*img->height;
	int p=0;
	while (p<len) {
		int num=len-p;
		if (num>32) num=32;
		putchar(0xc0+(num-1));
		int n;
		for (n=0;n<num;n++) {
			putchar(img->img[p]);
			p=p+1;
		}
	}
	refresh_badge();
}


int init_badge(image_t *img)
{
	putchar(BDG_INQRES); //Inquire resolution
	fflush(stdout);
	int w=0;
	int h=0;
	int c;
	while (gc()!=BDG_STX);
	c=gc();
	if (c!='B') return -1;
	c=gc();
	if (c!=' ') return -1;
	while ( ((c=gc())>='0') && (c<='9') ) w=w*10+c-'0';
	if (c!=' ') return -1;
	while ( ((c=gc())>='0') && (c<='9') ) h=h*10+c-'0';
	if (c!=BDG_ETX) return -1;
	return create_image(img, w,h);
}


void copy_image(image_t *src, image_t *dest, int sx, int sy)
{
	int x,y;
	for (y=0;y<dest->height;y++)
	for (x=0;x<dest->width;x++) {
		uint8_t c=0;
		if ( ((sx+x)<src->width) && ((sy+y)<src->height) ) {
			int p=(sx+x)+(sy+y)*src->width;
			c=src->img[p];
		}
		dest->img[x+y*dest->width]=c;
	}
}


